/*-----------------------------------------------------------------*/
/*! 
  \file omlfileio.cxx
  \internal
  \brief write/read to file descriptor (fd) or socket
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#if HAVE_CONFIG_H
#include "scscpconfig.h"
#endif /*HAVE_CONFIG_H*/
#include <stdio.h>
#include <sys/types.h>
#if HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#if HAVE_SYS_UIO_H
#include <sys/uio.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "scscptypes.h"
#include "scscpinternal.h"
#include "omlfileio.h"
#include "scscpdebug.h"


/*-----------------------------------------------------------------*/
/*! write the buffer buf of length len (bytes) to a file or socket
  return false on error
  @param buf (in) write buffer
  @param len (in) length of buf
*/
/*-----------------------------------------------------------------*/
bool OML_Fileio::write(const char *buf, size_t len)
{
 if (m_issocket)
 {
  return (::send(m_fd, buf, len, 0)!=SOCKET_ERROR);
 }
 else
 {
  return (::write(m_fd, buf, len)!=-1);
 }
}

/*-----------------------------------------------------------------*/
/*! read a buffer buf of length len (bytes) from a file or socket
 return the number of readen bytes 
 return -1 on error
  @param buf (out) read buffer
  @param len (in) length of buf
*/
/*-----------------------------------------------------------------*/
int OML_Fileio::read(char *buf, size_t len)
{
 if (m_issocket)
 {
  return ::recv(m_fd, buf, len, 0);
 }
 else
 {
  return ::read(m_fd, buf, len);
 }
}
