/*-----------------------------------------------------------------*/
/*! 
  \file omlfileoutputstream.h
  \brief Open Math Output Stream with file descriptor (int) definition.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __OMLFILEOUTPUTSTREAM_H__
#define __OMLFILEOUTPUTSTREAM_H__

#include "omloutputstream.h"
#include "omlfileio.h"

/*-----------------------------------------------------------------*/
/*!
    @class OML_Fileoutputstream
    \brief  classe de base pour gerer un flux sortant vers un 
    descripteur de fichier de type int en  Open Math
*/
/*-----------------------------------------------------------------*/
class OML_Fileoutputstream : public OML_outputstream
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   // ! use the file descriptor fd (return by open for example)
   OML_Fileoutputstream() : OML_outputstream(), m_fd() {};
   OML_Fileoutputstream(int fd, bool bsocket) : OML_outputstream(), m_fd(fd, bsocket) {};
   virtual ~OML_Fileoutputstream() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! attach the stream to an opened file descriptor
   void SetFile(int fd) { m_fd.SetFile(fd, false); };
   void SetSocket(int fd) { m_fd.SetFile(fd, true); };

   OML_Fileio& GetFileio() { return m_fd; };
    
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! fush the buffer to stream
   virtual bool Flush() { return WriteChars("\n"); };
   //! write characters to stream
   virtual bool WriteChars(const_OMLchar data);
   //! write characters to stream
   virtual OML_outputstream& operator<<(const_OMLchar data);
   //! write size_t to stream
   virtual OML_outputstream& operator<<(size_t data);
   //! write int to stream
   virtual OML_outputstream& operator<<(int data);
   //! write long long to stream
   virtual OML_outputstream& operator<<(long long int data);
   //! write long to stream
   virtual OML_outputstream& operator<<(long int data);
   
private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   OML_Fileio m_fd; //!< file descriptor
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! set flag of errors to true if value = false
   void CheckErrors(bool value)
   {
    if (!value) SetError();
   }
   
   ////////////////////////////////////////////////////////////////////////////
   // default values
   ////////////////////////////////////////////////////////////////////////////
};


#endif /*__OMLFILEOUTPUTSTREAM_H__*/
