/*-----------------------------------------------------------------*/
/*! 
  \file omlmpioutputstream.h
  \brief Open Math Output Stream with mpi definition.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __OMLMPIOUTPUTSTREAM_H__
#define __OMLMPIOUTPUTSTREAM_H__

#include "omloutputstream.h"
#include "omlmpistream.h"

/*-----------------------------------------------------------------*/
/* macros locales */
/*-----------------------------------------------------------------*/
/*! size of the buffer */
#define OML_MPI_BUFFERSIZE 3900

/*-----------------------------------------------------------------*/
/*!
    @class OML_MPIoutputstream
    \brief  classe de base pour gerer un flux sortant vers un 
    descripteur de fichier de type int en  Open Math
*/
/*-----------------------------------------------------------------*/
class OML_MPIoutputstream : public OML_outputstream, public OML_MPIstream
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   // ! use the file descriptor fd (return by open for example)
   OML_MPIoutputstream() : OML_outputstream(), OML_MPIstream(), m_bufferusage(0) {};
   OML_MPIoutputstream(int src, int tag, MPI_communicator comm = MPI_COMM_WORLD) : OML_outputstream(), OML_MPIstream(src, tag, comm), m_bufferusage(0) {};
   virtual ~OML_MPIoutputstream() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! fush the buffer to stream
   virtual bool Flush();
   //! write characters to stream
   virtual bool WriteChars(const_OMLchar data);
   //! write characters to stream
   virtual OML_outputstream& operator<<(const_OMLchar data);
   //! write size_t to stream
   virtual OML_outputstream& operator<<(size_t data);
   //! write int to stream
   virtual OML_outputstream& operator<<(int data);
   //! write long long to stream
   virtual OML_outputstream& operator<<(long long int data);
   //! write long to stream
   virtual OML_outputstream& operator<<(long int data);
   
private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   char m_buffer[OML_MPI_BUFFERSIZE]; //! buffer to reduce communication
   size_t m_bufferusage; //!< number of valid characters in m_buffer
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! set flag of errors to true if value = -1
   template<typename T> void CheckErrors(T value)
   {
    if (value==-1) SetError();
   }
   //! send a buffer preceded by th length of the buffer
   bool SendMPI(int len, char *buffer);
};


#endif /*__OMLMPIOUTPUTSTREAM_H__*/
