/*-----------------------------------------------------------------*/
/*! 
  \file omlmpistream.h
  \brief Open Math  Stream with mpi stream.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __OMLMPISTREAM_H__
#define __OMLMPISTREAM_H__

#include "scscpsetup.h"

#if SCSCP_HAVE_MPI
//add define to workaround mpich compilation problem
#define MPICH_IGNORE_CXX_SEEK 
#define MPICH_SKIP_MPICXX
#define MPIBULL_IGNORE_CXX_SEEK
#define MPIBULL_SKIP_MPICXX
#include <mpi.h>
#endif /*SCSCP_HAVE_MPI*/

/*-----------------------------------------------------------------*/
/*!
    @class OML_MPIstream
    \brief  classe de base pour gerer un flux MPI
*/
/*-----------------------------------------------------------------*/
class OML_MPIstream
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
#if SCSCP_HAVE_MPI
   typedef MPI_Comm MPI_communicator;
#else
   typedef int MPI_communicator;
#define    MPI_COMM_WORLD 0
#endif
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   OML_MPIstream() : m_dst(-1), m_tag(-1) {};
   OML_MPIstream(int dst, int tag, MPI_communicator comm) 
               :  m_dst(dst), m_comm(comm), m_tag(tag) {};
   ~OML_MPIstream() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   // return destination mpi
   int GetDst() const { return m_dst; };
   // return source mpi
   int GetSrc() const { return m_dst; };
   // return tag mpi
   int GetTag() const { return m_tag; };
   // return communicator mpi
   MPI_communicator GetComm() const { return m_comm; };
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   int m_dst; //!< destination id in MPI
   MPI_communicator m_comm; //!< communicator in MPI
   int m_tag; //!< tag id in MPI
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
};


#endif /*__OMLMPISTREAM_H__*/
