/*-----------------------------------------------------------------*/
/*! 
  \file omlomi.cxx
  \brief Open Math Integer object (OMI) implementation.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "omloutputstream.h"
#include "omlomi.h"
#include "scscpdebug.h"

/***************************************************************************/
/* macros locales aux fichiers 										   */
/***************************************************************************/
/*! conversion de xmlchar en char */
#define xmlChar2char(x) ((char*)(x))


/*-----------------------------------------------------------------*/
/*! write an OMI to output stream
  @param stream (inout) output stream
  @param integer (in) integer to write 
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Write(OML_outputstream& stream, size_t integer)
{
 stream.WriteBeginTagOMI();
 stream<<integer;
 return stream.WriteEndTagOMI();
}

/*-----------------------------------------------------------------*/
/*! write an OMI to output stream
  @param stream (inout) output stream
  @param integer (in) integer to write 
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Write(OML_outputstream& stream, long long integer)
{
 stream.WriteBeginTagOMI();
 stream<<integer;
 return stream.WriteEndTagOMI();
}

/*-----------------------------------------------------------------*/
/*! write an OMI to output stream
  @param stream (inout) output stream
  @param integer (in) integer to write 
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Write(OML_outputstream& stream, long integer)
{
 stream.WriteBeginTagOMI();
 stream<<integer;
 return stream.WriteEndTagOMI();
}
/*-----------------------------------------------------------------*/
/*! write an OMI to output stream
  @param stream (inout) output stream
  @param integer (in) integer to write 
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Write(OML_outputstream& stream, int integer)
{
 stream.WriteBeginTagOMI();
 stream<<integer;
 return stream.WriteEndTagOMI();
}
  
/*-----------------------------------------------------------------*/
/*! write an OMI to output stream
  @param stream (inout) output stream
  @param integer (in) integer to write 
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Write(OML_outputstream& stream, const char* integer)
{
 stream.WriteBeginTagOMI();
 stream<<integer;
 return stream.WriteEndTagOMI();
}

/*-----------------------------------------------------------------*/
/*! read an OMI from input stream and update the iterator
  @param iter (inout) iterator on the input stream
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Read(OML_inputstream::iterator& iter)
{
  bool bres = true;
  SCSCP_getdebugger()<<"OML_OMI::Read() - entree\n";
  
  m_value = xmlChar2char(xmlNodeGetContent(iter->children));
  OML_inputstream_iterator_next(iter);
  
  SCSCP_getdebugger()<<"OML_OMI::Read() - sortie et retourne "<<bres<<"\n";
  return bres;
}
   
/*-----------------------------------------------------------------*/
/*! return the integer value
 return false on error
  @param integer (out) integer 
*/
/*-----------------------------------------------------------------*/
/*bool OML_OMI::Get(unsigned long int& integer)
{
 if (m_value)
 {
  errno = 0;
  integer = strtoul(m_value, (char **)NULL, 10);
  return (errno==0);
 }
 else
 {
  integer = 0;
  return false;
 }
}*/

/*-----------------------------------------------------------------*/
/*! return the integer value
 return false on error
  @param integer (out) integer 
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Get(size_t& integer)
{
 if (m_value)
 {
  errno = 0;
  unsigned long lval = strtol(m_value, (char **)NULL, 10);
  integer = (size_t)lval;
  return (errno==0 && ((unsigned long)integer)==lval);
 }
 else
 {
  integer = 0;
  return false;
 }
}

/*-----------------------------------------------------------------*/
/*! return the integer value
 return false on error
  @param integer (out) integer 
*/
/*-----------------------------------------------------------------*/
bool OML_OMI::Get(int& integer)
{
 if (m_value)
 {
  errno = 0;
  long lval = strtol(m_value, (char **)NULL, 10);
  integer = (int)lval;
  return (errno==0 && ((long)integer)==lval);
 }
 else
 {
  integer = 0;
  return false;
 }
}
