/*-----------------------------------------------------------------*/
/*! 
  \file omloms.h
  \brief Open Math Integer object (OMS) definition.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __OMLOMS_H__
#define __OMLOMS_H__

#include "omlinputstream.h"

/*-----------------------------------------------------------------*/
/*!
    @class OML_OMS
    \brief  classe de base pour gerer les symboles Open Math
*/
/*-----------------------------------------------------------------*/
class OML_OMS
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   typedef OML_inputstream::const_OMLchar const_OMLchar;
   typedef OML_inputstream::OMLchar OMLchar;
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   OML_OMS();
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! write an OMS to output stream
   static bool Write(OML_outputstream& stream, OML_outputstream::const_OMLchar cdname, OML_outputstream::const_OMLchar name);
   //! read an OMS from input stream
   bool Read(OML_inputstream::iterator& iter);
   //! return the CD name
   const_OMLchar GetCD() const { return m_cdname; };
   //!return the name of the symbol
   const_OMLchar GetName() const { return m_name; };
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   OMLchar m_cdname; //!< name of the CD
   OMLchar m_name;   //!< name of the symbol
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! set the CD name
   void SetCD(OMLchar cd)  { m_cdname = cd; };
   //! set the symbol name
   void SetName(OMLchar name)  { m_name = name; };
   
   ////////////////////////////////////////////////////////////////////////////
   // default values
   ////////////////////////////////////////////////////////////////////////////
};


#endif /*__OMLOMS_H__*/
