/*-----------------------------------------------------------------*/
/*! 
  \file OMLOMV.cxx
  \brief Open Math variable object (OMV) implementation.
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "omloutputstream.h"
#include "omlomv.h"
#include "scscpdebug.h"

/***************************************************************************/
/* macros locales aux fichiers 										   */
/***************************************************************************/
/*! conversion de xmlchar en char */
#define xmlChar2char(x) ((char*)(x))


/*-----------------------------------------------------------------*/
/*! write an OMV to output stream
  @param stream (inout) output stream
  @param value (in) value to write 
*/
/*-----------------------------------------------------------------*/
bool OML_OMV::Write(OML_outputstream& stream, const_OMLchar value)
{
 return stream.WriteBeginEndTagOMV(value);
}
  
/*-----------------------------------------------------------------*/
/*! read an OMV to input stream
  @param iter (inout) iterator
  @param value   (out) read value
*/
/*-----------------------------------------------------------------*/
bool OML_OMV::Read(OML_inputstream::iterator& iter, OML_inputstream::const_OMLchar& value)
{
  bool bres = true;
  SCSCP_getdebugger()<<"OML_OMV::Read() - entree\n";
  
  value = OML_FindAttributes(iter, "name");
  if (value==NULL && iter->children!=NULL)
  {
   value = xmlChar2char(xmlNodeGetContent(iter->children));
  }
  if (value==NULL) bres = false;
  
  OML_inputstream_iterator_next(iter);
  
  SCSCP_getdebugger()<<"OML_OMV::Read(,"<<value<<") - sortie et retourne "<<bres<<"\n";
  return bres;

}
