/*-----------------------------------------------------------------*/
/*! 
  \file scscpclient.cxx
  \brief SCSCP client
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This library is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/

#include <stdio.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "scscpclient.h"
#include "scscpdebug.h"
#include "scscpfileclient.h"
#define __SCSCP_WITHIN_SCSCP
#include "scscp.h"



/*-----------------------------------------------------------------*/
/*! clear the allowed version to be negiotiated to the client 
 @param client (inout) object to be cleared
 @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_sc_clearallowedversions(SCSCP_client* client, SCSCP_status* status)
{
   int res = 1;
   
   SCSCP_getdebugger()<<"SCSCP_sc_clearallowedversions() - entree\n";
   if (client==NULL)
   {
     SCSCP_status_seterror(status,  SCSCP_STATUS_CLIENTOBJECTNULL);
     res = 0;
   }
   else
   {
    /* clear the elements of versions */
    if (client->m_allowedversions!=NULL)
    {
     for(size_t cpt=0; cpt<client->m_allowedversionscount; cpt++)
     {
      free(client->m_allowedversions[cpt]);
     }
     free(client->m_allowedversions);
     client->m_allowedversions = NULL;
     client->m_allowedversionscount = 0;
    }
   }
   SCSCP_getdebugger()<<"SCSCP_sc_clearallowedversions() - sortie\n";
   return res;
}
