/*-----------------------------------------------------------------*/
/*! 
  \file scscpdebug.h
  \brief SCSCP debug
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPDEBUG_H__
#define __SCSCPDEBUG_H__

#include <stdio.h>

/*! define it if debug is disabled */
#define SCSCP_DISABLE_DEBUG

////////////////////////////////////////////////////////////////////////////
// types
////////////////////////////////////////////////////////////////////////////

#if defined (__cplusplus)

/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_debug
    \brief  classe de base pour gerer le debug de SCSCP
*/
/*-----------------------------------------------------------------*/
class SCSCP_debug
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! write debug message 
   const SCSCP_debug& operator<<(const char * msg) const 
   { 
 #ifndef SCSCP_DISABLE_DEBUG
    printf("%s", msg==NULL?"NULL":msg); 
 #endif
    return *this; 
   };

   //! write debug message 
   const SCSCP_debug& operator<<(const unsigned char * msg) const 
   { 
 #ifndef SCSCP_DISABLE_DEBUG
    printf("%s", msg); 
 #endif
    return *this; 
   };

   //! write debug message 
   const SCSCP_debug& operator<<(bool msg) const
   { 
 #ifndef SCSCP_DISABLE_DEBUG
     printf("%s", msg?"true":"false"); 
 #endif
    return *this; 
   };

   //! write debug message 
   const SCSCP_debug& operator<<(int msg) const
   { 
 #ifndef SCSCP_DISABLE_DEBUG
     printf("%d", msg); 
 #endif
    return *this; 
   };

   //! write debug message 
   const SCSCP_debug& operator<<(char msg) const
   { 
 #ifndef SCSCP_DISABLE_DEBUG
     printf("%c", msg); 
 #endif
    return *this; 
   };
   
#if 0
//! write debug message 
   const SCSCP_debug& operator<<(unsigned long int msg) const
   { 
 #ifndef SCSCP_DISABLE_DEBUG
     printf("%lu", msg); 
 #endif
    return *this; 
   };
#endif

   //! write debug message 
   const SCSCP_debug& operator<<(size_t msg) const
   { 
 #ifndef SCSCP_DISABLE_DEBUG
     printf("%llu", (unsigned long long int ) msg); 
 #endif
    return *this; 
   };

   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////


private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
};


/* return the default debugger */
const SCSCP_debug& SCSCP_getdebugger();

#endif /*defined (__cplusplus)*/

#if defined(SCSCP_DISABLE_DEBUG)
static inline void SCSCP_debugprint(const char* format, ...) { }
#else
#define SCSCP_debugprint printf
#endif

#endif /*__SCSCPDEBUG_H__*/
