/*-----------------------------------------------------------------*/
/*! 
  \file scscpfileclient.h
  \brief SCSCP client with file descriptor
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008, 2009, M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPFILECLIENT_H__
#define __SCSCPFILECLIENT_H__

#include "scscpclient.h"
#include "omlfileoutputstream.h"
#include "omlfileinputstream.h"

/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_Fileclient
    \brief  classe pour gerer les connexions clients SCSCP avec file descriptor
*/
/*-----------------------------------------------------------------*/
class SCSCP_Fileclient : public SCSCP_client
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_Fileclient() : m_fd(-1) {};
   virtual ~SCSCP_Fileclient() { Close(); };
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! connect to a server at the specified port
   bool Connect(const char *servername, int port, SCSCP_status* status);
   //! close the connexion to the server 
   void Close();
   //! return the socket descriptor
   int GetFileio() const { return m_fd; };
   //! Attach a socket
   void SetSocket(int port); 
   
   //!return the iterator on the stream
   OML_inputstream::iterator& GetIter() { return m_iter; };
 
   //! read a line from the input stream and fill buffer.
   bool ReadLine(char *buffer, int lenbuffer, SCSCP_status *status);
  
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! return the output stream with the connection
   virtual OML_outputstream* GetOutputStream() { return &m_OutStream; };
    //! return the input stream with the connection
   virtual OML_inputstream*  GetInputStream() { return &m_InStream; };

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   int m_fd; //!< socket descriptor
   OML_inputstream::iterator m_iter; //!< current iterator
   OML_Fileoutputstream m_OutStream; //!< flux sortant
   OML_Fileinputstream m_InStream;   //!< flux entrant
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //! read the CIM version from the server
   bool ReadCIMVersion(SCSCP_status* status);
   //! write the Connection information Message from the server
   bool ReadCIM(const char **supportedversion,SCSCP_status* status);
   //! write the CIM version to the server
   bool WriteCIMVersion(const char *supportedversion);
   //! get the socket
   int GetSocket() const { return m_fd; };
};



#endif /*__SCSCPFILECLIENT_H__*/
