/*-----------------------------------------------------------------*/
/*! 
  \file scscpinternal.h
  \brief SCSCP internal functions
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008, 2009,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr

   \bug M. GASTINEAU 20/04/09 : update for SCSCP 1.3
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#if !defined(__SCSCPINTERNAL_H__) 
#define __SCSCPINTERNAL_H__

#include <stdlib.h>

#if defined (__cplusplus)
#if !defined(__UTIL_H__)
inline void *operator new( size_t s ) { void* p=(void*)malloc(s); return p; }
/*! overload the operator delete */
inline void operator delete (void* p) { free(p); }
/*! overload the operator new[] */
inline void *operator new[] ( size_t s ) { void* p=(void*)malloc(s); return p; }
/*! overload the operator delete[] */
inline void operator delete[]  (void* p) { free(p); }
#endif 

class SCSCP_Fileclient;
class SCSCP_Fileserver;
class SCSCP_procedurecalloptions;
class SCSCP_optionsprocedurereturn;

typedef SCSCP_Fileclient* SCSCP_socketclient;
typedef SCSCP_Fileserver* SCSCP_socketserver;
typedef SCSCP_Fileclient* SCSCP_incomingclient;
typedef SCSCP_Fileclient* SCSCP_io;
typedef SCSCP_procedurecalloptions* SCSCP_calloptions;
typedef SCSCP_optionsprocedurereturn* SCSCP_returnoptions;

#else  /* (__cplusplus) */
typedef void* SCSCP_Fileserver;
typedef void* SCSCP_socketclient;
typedef void* SCSCP_socketserver;
typedef void* SCSCP_incomingclient;
typedef void* SCSCP_io;
typedef void* SCSCP_calloptions;
typedef void* SCSCP_returnoptions;

#endif /* (__cplusplus) */


int SCSCP_sc_checkNULL(SCSCP_socketclient* client, SCSCP_status* status);
int SCSCP_ss_checkNULL(SCSCP_Fileserver* server, SCSCP_status* status);
int SCSCP_io_writef(SCSCP_io* stream, SCSCP_status* status, const char *buffer, ...);
int SCSCP_io_ReadPI(SCSCP_io* stream, SCSCP_msgtype* msgtype, char *bufferPI, SCSCP_status* status);

/*! generate and set the procedure call ID (prefixed by libSCSCP:) */
int SCSCP_co_generateunique_callid(SCSCP_calloptions* options, SCSCP_status* status); 


int SCSCP_initsocket();
/* macros pour compatibilite des sockets entre unix et windows */
#if !HAVE_WINDOWS_H || !defined(INVALID_SOCKET)
#define INVALID_SOCKET (-1) 
#define SOCKET_ERROR (-1) 
#define closesocket(sock) close(sock)
#endif

#if defined (__cplusplus)
extern "C" {
#endif /* (__cplusplus) */

/*! allocate memory of ncount bytes (similar to malloc) */
void* SCSCP_malloc(size_t ncount, SCSCP_status* status);
/*! duplicate a string (similar to strdup) */
char* SCSCP_strdup(const char*, SCSCP_status* status);

#if !HAVE_SNPRINTF
int SCSCP_snprintf(char *string, size_t n, const char *format, ...);
#define snprintf SCSCP_snprintf
#endif 

char* SCSCP_findPIstring(char *c, const char *versionname);


/*! set the error to value */
void SCSCP_status_seterror(SCSCP_status* status, int value);
/*! set the error to value  : EXECFAILED */
void SCSCP_status_setexecfailed(SCSCP_status* status, const char *cdname, const char *symbolname, const char *msg);


//forward declaration
#ifndef LIBXML_DOTTED_VERSION
typedef struct _xmlDoc xmlDoc;
typedef xmlDoc* xmlDocPtr;
typedef struct _xmlNode* xmlNodePtr;
#endif

typedef struct _xmlNode* SCSCP_xmlnodeptr;
typedef struct _xmlAttr* SCSCP_xmlattrptr;


/*-----------------------------------------------------------------*/
//! buffer for len for scscp Processing instruction
/*-----------------------------------------------------------------*/
#define SCSCP_PI_MAXLENBUFFER 4096


#if defined (__cplusplus)
}
#endif /* (__cplusplus) */

#endif /*__SCSCPINTERNAL_H__*/
