/*-----------------------------------------------------------------*/
/*! 
  \file scscpoptions.cxx
  \brief SCSCP options for messages
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008, 2009,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr

  \bug M. GASTINEAU 20/04/09 : update for SCSCP 1.3
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#if HAVE_CONFIG_H
#include "scscpconfig.h"
#endif /*HAVE_CONFIG_H*/
#include <stdio.h>
#if HAVE_STRING_H
#include <string.h>
#endif
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "scscptypes.h"
#include "scscpinternal.h"
#include "scscpoptions.h"
#include "scscpdebug.h"
#include "omloms.h"
#include "scscptags.h"
#define __SCSCP_WITHIN_SCSCP
#include "scscp.h"

/*-----------------------------------------------------------------*/
/*!
    write the object to the stream
   @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool SCSCP_optionsprocedurereturn::Write(OML_outputstream& stream) const
{
 SCSCP_getdebugger()<<"\nSCSCP_optionsprocedurereturn::Write() - entree\n";
 
 stream.WriteBeginTagOMATP();
 //write call id
 GetCallID().Write(stream);
 //write runtime limit
 if (m_set_used_runtime) stream.WritePairOMSOMI(SCSCP_tags::cdname, SCSCP_tags::info_runtime, GetRuntime());
 //write used memory
 if (m_set_used_memory) stream.WritePairOMSOMI(SCSCP_tags::cdname, SCSCP_tags::info_memory, GetUsedMemory());
 //write information message
 if (m_set_used_message) stream.WritePairOMSOMSTR(SCSCP_tags::cdname, SCSCP_tags::info_message, m_message);
 
 bool res = stream.WriteEndTagOMATP();
 
 SCSCP_getdebugger()<<"\nSCSCP_optionsprocedurereturn::Write() - sortie et retourne "<<res<<"\n";
 return res;
}


/*-----------------------------------------------------------------*/
//! constructor
/*-----------------------------------------------------------------*/
SCSCP_optionsprocedurereturn::SCSCP_optionsprocedurereturn()
                          :  m_set_used_runtime(false), m_set_used_memory(false), m_set_used_message(false)
{
}

/*-----------------------------------------------------------------*/
/*! \internal set the information message infomessage to the return options p_pro
 @param p_pro (inout) return options
 @param infomessage (inout) message 
*/
/*-----------------------------------------------------------------*/
void SCSCP_optionsprocedurereturn_SetInfoMessage(SCSCP_optionsprocedurereturn* p_pro,const_SCSCP_string infomessage)
{
  p_pro->m_message = infomessage; 
  p_pro->m_set_used_message = true; 
}


/*-----------------------------------------------------------------*/
/*! \internal return true if the information message of p_pro has been set.
 @param p_pro (in) return options
*/
/*-----------------------------------------------------------------*/
bool SCSCP_optionsprocedurereturn_IsMessageValid(const SCSCP_optionsprocedurereturn* p_pro)
{
 return p_pro->m_set_used_message;
}

/*-----------------------------------------------------------------*/
/*! \internal return the information message of the return options
 @param p_pro (inout) return options
*/
/*-----------------------------------------------------------------*/
const char* SCSCP_optionsprocedurereturn_GetInfoMessage(const SCSCP_optionsprocedurereturn* p_pro)
{
  return p_pro->m_message; 
}


/*-----------------------------------------------------------------*/
/*! read the object from the stream
   @param stream (inout) input stream
   @param iterchild (out) iterator on the input stream
   @param omsread (out) type of message
   @param status (inout) status information 
*/
/*-----------------------------------------------------------------*/
bool SCSCP_optionsprocedurereturn::Read(OML_inputstream& stream, 
                                        OML_inputstream::iterator& iterchild, 
                                        eTypeReception& omsread,
                                        SCSCP_status* status)
{
 bool res;
 SCSCP_getdebugger()<<"SCSCP_optionsprocedurereturn::Read() - entree\n";
 
 res = stream.ReadXmlDoc(status);
 if (res)
 {
  SCSCP_getdebugger()<<"SCSCP_optionsprocedurereturn::Read() - ReadXmlDoc returns true\n";
  OML_inputstream::iterator iter = stream.getiterator();
  res = stream.ReadTagOMOBJ(iter);
  
  iter = stream.GoToChild(iter);
  
  // parse attributes
  res = ReadOptions(stream, iter);
  //parse the name of the procedure
  res = stream.ReadTagOMA(iter, false);
  if (res)
  {
    OML_OMS oms;
    iterchild = stream.GoToChild(iter);
    res = oms.Read(iterchild);
    if (strcmp(oms.GetName(),SCSCP_tags::procedure_completed)==0 &&
        strcmp(oms.GetCD(),SCSCP_tags::cdname)==0) omsread = SCSCP_msgtype_ProcedureCompleted;
    else if (strcmp(oms.GetName(),SCSCP_tags::procedure_terminated)==0 &&
        strcmp(oms.GetCD(),SCSCP_tags::cdname)==0) omsread = SCSCP_msgtype_ProcedureTerminated;
    else 
    {
     SCSCP_getdebugger()<<"SCSCP_optionsprocedurereturn::Read() - symbol non reconnu "<<oms.GetCD()<<" "<<oms.GetName()<<"\n";
     res = false;
    }
  }
 }
 SCSCP_getdebugger()<<"SCSCP_optionsprocedurereturn::Read() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! read the options from the stream
   @param stream (inout) input stream
   @param iter (inout) iterator
*/
/*-----------------------------------------------------------------*/
bool SCSCP_optionsprocedurereturn::ReadOptions(OML_inputstream& stream,  OML_inputstream::iterator& iter)
{
 bool res, res1, res3;
 SCSCP_getdebugger()<<"SCSCP_optionsprocedurereturn::ReadOptions() - entree\n";
 
 OML_inputstream::iterator iterend, iter2, iter3;
 SCSCP_getdebugger()<<"SCSCP_optionsprocedurereturn::Read() - entree\n";
 
 iter2 = iter;
 res3 = stream.ReadTagOMATTR(iter);
 iter3 = iter;
 res = stream.ReadTagOMATP(iter);
 iterend = iter;
 /* try to read all attributes */
 if (res)
 {
  const_SCSCP_string infomessage;
  OML_inputstream::iterator iterchild = iter->children;
  res  = m_callid.Read(stream, iterchild);
  SCSCP_time timer;
  res1 = stream.ReadPairOMSOMI(iterchild, SCSCP_tags::cdname, SCSCP_tags::info_runtime, timer); 
  if (res1) SetRuntime(timer);
  //read information memory
  SCSCP_memory mem;
  res1 = stream.ReadPairOMSOMI(iterchild, SCSCP_tags::cdname, SCSCP_tags::info_memory, mem); 
  if (res1) SetUsedMemory(mem);
  //read information message
  res1 = stream.ReadPairOMSOMSTR(iterchild, SCSCP_tags::cdname, SCSCP_tags::info_message, infomessage); 
  if (res1) SCSCP_optionsprocedurereturn_SetInfoMessage(this, infomessage);
  iter = iterend;
 }
 else if (res3)
 {
  res = true;
  iter = iter3->children; 
 }
 else
 {
  res = true;
  iter = iter2; 
 }
 SCSCP_getdebugger()<<"SCSCP_optionsprocedurereturn::ReadOptions() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! checks if options is null and returns 0 on error
  @param options (in) object to be checked
  @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
static int SCSCP_ro_checkNULL(SCSCP_returnoptions* options, SCSCP_status* status)
{
   int res = 1;
   
   if (options==NULL || *options==NULL)
   {
     SCSCP_status_seterror(status,  SCSCP_STATUS_RETURNOPTIONSOBJECTNULL);
     res = 0;
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*! initialize the object options and set status if an error occurs
   @param options (out) options
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_init(SCSCP_returnoptions* options, SCSCP_status* status)
{
   int res = 1;
   
   if (options==NULL)
   {
     SCSCP_status_seterror(status, SCSCP_STATUS_RETURNOPTIONSOBJECTNULL);
     res = 0;
   }
   else
   {
    *options = new SCSCP_optionsprocedurereturn;
    if (*options==NULL) 
    {
     SCSCP_status_seterror(status, SCSCP_STATUS_NOMEM);
     res = 0;
    }
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*!  clear the object options previously initialized by SCSCP_ro_init
   @param options (inout) options
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_clear(SCSCP_returnoptions* options, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res) 
   {
    delete (*options);
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*! set the procedure call ID (buffer can't be released before calling SCSCP_ro_clear)
   @param options (inout) options
   @param buffer (in) string buffer
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_set_callid(SCSCP_returnoptions* options, const char *buffer, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res)  (*options)->SetCallIDConst(buffer);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the procedure call ID
   @param options (inout) options
   @param buffer (out) string buffer
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_get_callid(SCSCP_returnoptions* options, const char **buffer, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res)  *buffer = SCSCP_call_id_Get(&((*options)->GetCallID()));
   return res;
}

/*-----------------------------------------------------------------*/
/*! set the runtime usage
   @param options (inout) options
   @param time (in) time usage
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_set_runtime(SCSCP_returnoptions* options, size_t time, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res)  (*options)->SetRuntime(time);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the runtime usage
  if the runtime usage is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNRUNTIME
   @param options (inout) options
   @param time (out) time usage. can't be NULL
   @param status (inout) status error
       if the runtime usage is unknown
           => status is set to SCSCP_STATUS_USAGEUNKNOWNRUNTIME 
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_get_runtime(SCSCP_returnoptions* options, size_t* time, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res) 
   {
    res = (*options)->IsRuntimeValid();
    if (!res) 
    {
     SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNRUNTIME);
     *time = 0;
    }
    else
    {
     *time = (*options)->GetRuntime();
    }
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*! set the memory usage
   @param options (inout) options
   @param memsize (in) memory usage
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_set_memory(SCSCP_returnoptions* options, size_t memsize, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res)  (*options)->SetUsedMemory(memsize);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the memory usage
  if the runtime usage is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNMEM
   @param options (inout) options
   @param memsize (out) memory usage. can't be NULL
   @param status (inout) status error
       if the runtime usage is unknown
           => status is set to SCSCP_STATUS_USAGEUNKNOWNMEM 
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_get_memory(SCSCP_returnoptions* options, size_t* memsize, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res) 
   {
    res = (*options)->IsMemoryValid();
    if (!res) 
    {
     SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNMEM);
     *memsize = 0;
    }
    else
    {
     *memsize = (*options)->GetUsedMemory();
    }
   }
   return res;
}

/*-----------------------------------------------------------------*/
/*! set the information message
   @param options (inout) options
   @param buffer (in) string buffer
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_set_message(SCSCP_returnoptions* options, const char *buffer, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res) SCSCP_optionsprocedurereturn_SetInfoMessage(*options, buffer);
   return res;
}

/*-----------------------------------------------------------------*/
/*! get the information message
  if the runtime usage is unknown => it returns 0 
     and status is set to  SCSCP_STATUS_USAGEUNKNOWNMESSAGE
   @param options (inout) options
   @param buffer (out) string buffer
   @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_ro_get_message(SCSCP_returnoptions* options, const char **buffer, SCSCP_status* status)
{
   int res = SCSCP_ro_checkNULL(options, status);
   if (res) 
   {
    res = SCSCP_optionsprocedurereturn_IsMessageValid(*options);
    if (!res) 
    {
     SCSCP_status_seterror(status, SCSCP_STATUS_USAGEUNKNOWNMESSAGE);
     *buffer = NULL;
    }
    else
    {
     *buffer = SCSCP_optionsprocedurereturn_GetInfoMessage(*options);
    } 
   }
   return res;
}

