/*-----------------------------------------------------------------*/
/*! 
  \file scscpoptions.h
  \brief SCSCP options for messages
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008, 2009,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr

   \bug M. GASTINEAU 20/04/09 : update for SCSCP 1.3
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPOPTIONS_H__
#define __SCSCPOPTIONS_H__

#include "scscptypes.h"
#include "scscpcallid.h"


/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_optionsprocedurereturn
    \brief  classe pour gerer les options communes des  messages  procedurecompleted et proceduretermianted
*/
/*-----------------------------------------------------------------*/
class SCSCP_optionsprocedurereturn
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////

   ////////////////////////////////////////////////////////////////////////////
   // enumeration
   ////////////////////////////////////////////////////////////////////////////
   typedef SCSCP_msgtype eTypeReception;
   
   
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_optionsprocedurereturn();
   virtual ~SCSCP_optionsprocedurereturn() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! return call ID
   const SCSCP_call_id& GetCallID() const { return m_callid; };
   //! set call ID
   void SetCallIDConst(const char *buffer) { SCSCP_call_id_Set(&m_callid, buffer); };
   
   //! return the runtime used
   const SCSCP_time& GetRuntime() const { return m_runtime; };
   //! set the runtime used
   void SetRuntime(SCSCP_time time)  { m_runtime = time; m_set_used_runtime = true; };
   //! return true if runtime used is valid
   bool IsRuntimeValid() const { return m_set_used_runtime; };
   
   //! return the used memory  in bytes
   const SCSCP_memory& GetUsedMemory() const { return m_used_memory; };
   //! set the used memory  in bytes
   void SetUsedMemory(SCSCP_memory mem)  { m_used_memory = mem; m_set_used_memory = true; };
    //! return true if memory used is valid
   bool IsMemoryValid() const { return m_set_used_memory; };
  
   //!read the header message and return the type
   bool Read(OML_inputstream& stream,  OML_inputstream::iterator& iterchild, 
             eTypeReception& omsread,  SCSCP_status* status);

   //! friend function
   friend void SCSCP_optionsprocedurereturn_SetInfoMessage(SCSCP_optionsprocedurereturn* p_pro,const_SCSCP_string infomessage);
   friend bool SCSCP_optionsprocedurereturn_IsMessageValid(const SCSCP_optionsprocedurereturn* p_pro);
   friend const char* SCSCP_optionsprocedurereturn_GetInfoMessage(const SCSCP_optionsprocedurereturn* p_pro);

   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! write the object to the stream
   virtual bool Write(OML_outputstream& stream) const;

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_call_id       m_callid; //!< call identifier
   SCSCP_time          m_runtime; //!< runtime  in milliseconds
   SCSCP_memory        m_used_memory; //!< used memory  in bytes
   const_SCSCP_string  m_message; //!< information message
    
   // specify if attributes are set
   bool m_set_used_runtime; //!< = true => m_runtime is valid
   bool m_set_used_memory;  //!< = true => m_used_memory is valid
   bool m_set_used_message; //!< = true => m_message is valid
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   //!read the options
   bool ReadOptions(OML_inputstream& stream,  OML_inputstream::iterator& iterchild);
   
};

void SCSCP_optionsprocedurereturn_SetInfoMessage(SCSCP_optionsprocedurereturn* p_pro,const_SCSCP_string infomessage);
bool SCSCP_optionsprocedurereturn_IsMessageValid(const SCSCP_optionsprocedurereturn* p_pro);
const char* SCSCP_optionsprocedurereturn_GetInfoMessage(const SCSCP_optionsprocedurereturn* p_pro);

#endif /*__SCSCPOPTIONS_H__*/
