/*-----------------------------------------------------------------*/
/*! 
  \file scscpprocedurecall.h
  \brief SCSCP message : procedure call.
   class SCSCP_procedurecalloptions, SCSCP_procedurecall
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008, 2009,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr

  \bug M. GASTINEAU 12/09/08 : update for SCSCP 1.2
  \bug M. GASTINEAU 20/04/09 : update for SCSCP 1.3
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This library is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPPROCEDURECALL_H__
#define __SCSCPPROCEDURECALL_H__

#include "scscptypes.h"
#include "omloutputstream.h"
#include "omlinputstream.h"
#include "scscpcallid.h"




/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_procedurecalloptions
    \brief  classe pour gerer les options du  message Procedure call
*/
/*-----------------------------------------------------------------*/
class SCSCP_procedurecalloptions
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   //! debug level type
   typedef int SCSCP_debuglevel;
   
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_procedurecalloptions();
   virtual ~SCSCP_procedurecalloptions() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   //! return call ID
   const SCSCP_call_id& GetCallID() const { return m_callid; };
   //! set call ID
   void SetCallIDConst(const char *buffer) { SCSCP_call_id_Set(&m_callid, buffer); m_set_callid = true; };
   //! set call ID : buffer will be destroyed by this object
   void SetCallID(char *buffer) { SCSCP_call_id_SetWritable(&m_callid, buffer); m_set_callid = true; };
   //! return the runtime limit
   const SCSCP_time& GetRuntimeLimit() const { return m_runtime_limit; };
   //! set the runtime limit
   void SetRuntimeLimit(SCSCP_time limit)  { m_runtime_limit = limit; m_set_runtime_limit = true; };
   //! is the runtime limit valid
   bool IsRuntimeLimitValid()   const { return m_set_runtime_limit; };
   
   //! return the minimal memory required in bytes
   const SCSCP_memory& GetMinimalMemory() const { return m_minimal_memory; };
   //! set the minimal memory required in bytes
   void SetMinimalMemory(SCSCP_memory limit)  { m_minimal_memory = limit; m_set_minimal_memory = true; };
   //! is the minimal memory  valid ?
   bool IsMinimalMemoryValid()   const { return m_set_minimal_memory; };
   
   //! return the maximal memory required in bytes
   const SCSCP_memory& GetMaximalMemory() const { return m_maximal_memory; };
   //! set the maximal memory required in bytes
   void SetMaximalMemory(SCSCP_memory limit)  { m_maximal_memory = limit; m_set_maximal_memory = true; };
   //! is the maximal memory valid ?
   bool IsMaximalMemoryValid()   const { return m_set_maximal_memory; };
   
   //! return the debug level
   const SCSCP_debuglevel& GetDebugLevel() const { return m_debuglevel; };
   //! set the debug level
   void SetDebugLevel(SCSCP_debuglevel level)  { m_debuglevel = level; m_set_debuglevel = true; };
   //! is the debug level valid ?
   bool IsDebugLevelValid()   const { return m_set_debuglevel; };
   
   //! return the return type
   const SCSCP_option_return& GetReturnType() const { return m_returntype; };
   //! set the return type
   void SetReturnType(SCSCP_option_return type)  { m_returntype = type; m_set_returntype = true; };
   //! is the return type valid ?
   bool IsReturnTypeValid()   const { return m_set_returntype; };
   //return true if return type is cookie
   bool IsReturnTypeCookie() const { return m_returntype==SCSCP_option_return_cookie; };
   //return true if return type is nothing
   bool IsReturnTypeNothing() const { return m_returntype==SCSCP_option_return_nothing; };
   
   //! display debug information
   void Debug() const;
   
   //! write the object to the stream
   int Write(OML_outputstream& stream, SCSCP_status* status) const;
   //! read the object to the stream
   bool Read(OML_inputstream& stream, OML_inputstream::iterator& iter);

   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_call_id       m_callid; //!< call identifier
   SCSCP_option_return m_returntype; //!< type of the return
   SCSCP_time          m_runtime_limit; //!< runtime limit in milliseconds
   SCSCP_memory        m_minimal_memory; //!< minimal memory required in bytes
   SCSCP_memory        m_maximal_memory; //!< maximal memory required in bytes
   SCSCP_debuglevel    m_debuglevel;     //!< debug level
   
   // specify if attributes are set
   bool m_set_callid; //! = true => m_callid is valid
   bool m_set_returntype; //! = true => m_returntype is valid
   bool m_set_runtime_limit; //! = true => m_runtime_limit is valid
   bool m_set_minimal_memory; //! = true => m_minimal_memory is valid
   bool m_set_maximal_memory; //! = true => m_maximal_memory is valid
   bool m_set_debuglevel; //! = true => m_debuglevel is valid
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   
};



/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_procedurearguments
    \brief  classe pour gerer les parametres du message Procedure call
*/
/*-----------------------------------------------------------------*/
class SCSCP_procedurearguments
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_procedurearguments() {};
   virtual ~SCSCP_procedurearguments() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////
   //! write the object to the stream
   virtual bool Write(OML_outputstream& stream) const { return true; };
   //! read the object from the stream
   virtual bool Read(OML_inputstream& stream, OML_inputstream::iterator iter, const_SCSCP_string procedurename) { return true; };


private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   
};

/*-----------------------------------------------------------------*/
/*!
    @class SCSCP_procedurecall
    \brief  classe pour gerer un message Procedure call
*/
/*-----------------------------------------------------------------*/
class SCSCP_procedurecall
{
 public:
   ////////////////////////////////////////////////////////////////////////////
   // types
   ////////////////////////////////////////////////////////////////////////////
   
   ////////////////////////////////////////////////////////////////////////////
   // constructeur destructeur
   ////////////////////////////////////////////////////////////////////////////
   SCSCP_procedurecall(const_SCSCP_string pCDName, const_SCSCP_string pProcedureName, SCSCP_procedurecalloptions& pProcedureOptions, SCSCP_procedurearguments& pProcedureArguments)
                     :m_ProcedureName(pProcedureName), m_CDName(pCDName), 
                      m_ProcedureOptions(pProcedureOptions), m_ProcedureArguments(pProcedureArguments) 
   {};
   SCSCP_procedurecall(SCSCP_procedurecalloptions& pProcedureOptions, SCSCP_procedurearguments& pProcedureArguments)
                     :m_ProcedureName(NULL), m_CDName(NULL),
                      m_ProcedureOptions(pProcedureOptions), m_ProcedureArguments(pProcedureArguments)
   {};
   virtual ~SCSCP_procedurecall() {};
   
   ////////////////////////////////////////////////////////////////////////////
   // operation
   ////////////////////////////////////////////////////////////////////////////
   // display debug information
   void Debug() const;
   //! return the name of the procedure
   const_SCSCP_string GetProcName() const { return m_ProcedureName; };
   //! return the name of the CD
   const_SCSCP_string GetCDName() const { return m_CDName; };
   //!return the call options
   const SCSCP_procedurecalloptions& GetCallOptions() const { return m_ProcedureOptions; };
   
   //! write the object to the stream
   bool Write(OML_outputstream& stream, SCSCP_status* status) const ;
   //! read the object from the stream
   bool Read(OML_inputstream& stream, SCSCP_status *status);

   //! read the header of the procedure call
   bool ReadHeader(OML_inputstream& stream, OML_inputstream::iterator& iter,
                   SCSCP_status* status);

   ////////////////////////////////////////////////////////////////////////////
   // operation surchargee
   ////////////////////////////////////////////////////////////////////////////

private:
   ////////////////////////////////////////////////////////////////////////////
   // attribut
   ////////////////////////////////////////////////////////////////////////////
   const_SCSCP_string m_ProcedureName; //!< name of the procedure
   const_SCSCP_string m_CDName;        //!< name of the CD
   SCSCP_procedurecalloptions& m_ProcedureOptions; //!< options of the procedure
   SCSCP_procedurearguments&   m_ProcedureArguments; //!< arguments of the procedure
   
   ////////////////////////////////////////////////////////////////////////////
   // operation privee
   ////////////////////////////////////////////////////////////////////////////
   
};

class SCSCP_Fileclient;
int SCSCP_procedurecall_Write(SCSCP_Fileclient* client, SCSCP_procedurecalloptions* options, 
                          int (*callbackwriteargs)(SCSCP_Fileclient** client, void *param, SCSCP_status* status),
                          void *param, SCSCP_status* status);

#endif /*__SCSCPPROCEDURECALL_H__*/
