/*-----------------------------------------------------------------*/
/*! 
  \file scscpprocedurecompleted.cxx
  \brief SCSCP message : procedure completed.
   class  SCSCP_procedurecompleted
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#include <stdio.h>
#if HAVE_STRING_H
#include <string.h>
#endif
#include "scscptypes.h"
#include "scscpinternal.h"
#include "scscpprocedurecompleted.h"
#include "scscpdebug.h"
#include "omloms.h"
#include "omlomr.h"
#include "omlomobj.h"
#include "scscptags.h"
#include "scscpfileclient.h"

/*-----------------------------------------------------------------*/
/*! constructor
   @param options (inout) options of the procedure completed
   @param args (inout) arguments of the procedure call 
*/
/*-----------------------------------------------------------------*/
SCSCP_procedurecompleted::SCSCP_procedurecompleted(SCSCP_procedurecompletedoptions& options, SCSCP_procedurearguments& args)
    : m_ProcedureOptions(options), m_Resultats(args), m_cookie(NULL)
{
}

/*-----------------------------------------------------------------*/
/*! write the object to the stream
   @param stream (inout) output stream
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedurecompleted::Write(OML_outputstream& stream) const
{
 SCSCP_getdebugger()<<"\nSCSCP_procedurecompleted::Write() - entree\n";
 
 stream.WriteSCSCPStartMsg();
 
 OML_OMOBJ::WriteBeginTag(stream);
 stream.WriteBeginTagOMATTR();
 
 //write the options
 m_ProcedureOptions.Write(stream);
 
 stream.WriteBeginTagOMA();
 
 //write procedure name  
 const_SCSCP_string cookie = GetReturnCookie();
 if (cookie)
 {
  stream.WritePairOMSOMR(SCSCP_tags::cdname, SCSCP_tags::procedure_completed,cookie);
 }
 else
 {
  OML_OMS::Write(stream, SCSCP_tags::cdname, SCSCP_tags::procedure_completed);
  //write returned object
  WriteObject(stream);
 }
 stream.WriteEndTagOMA();
 
 stream.WriteEndTagOMATTR();
 OML_OMOBJ::WriteEndTag(stream);
 bool res = stream.WriteSCSCPEndMsg();

 SCSCP_getdebugger()<<"\nSCSCP_procedurecompleted::Write() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! send the "procedure completed" message to the client. 
    The arguments  are written by callbackwriteargs
 @param client (inout) scscp client
 @param options (in) options of the call
 @param callbackwriteargs (in) callback function 
             to write the parameters of the procedure call
 @param param (inout) opaque data given to callbackwriteargs
 @param status (inout) status error
*/
/*-----------------------------------------------------------------*/
int SCSCP_procedurecompleted_Write(SCSCP_Fileclient* client, SCSCP_procedurecompletedoptions* options, 
                          int (*callbackwriteargs)(SCSCP_Fileclient** client, void *param, SCSCP_status* status),
                          void *param, SCSCP_status* status)
{
 SCSCP_getdebugger()<<"\nSCSCP_procedurecompleted_Write() - entree\n";
 
 OML_Fileoutputstream stream(client->GetFileio(), true);
 stream.WriteSCSCPStartMsg();
 
 OML_OMOBJ::WriteBeginTag(stream);
 stream.WriteBeginTagOMATTR();
 
 //write the options
 if (options) options->Write(stream);
 
 stream.WriteBeginTagOMA();
 
 //write procedure name  
 OML_OMS::Write(stream, SCSCP_tags::cdname, SCSCP_tags::procedure_completed);
 //write returned object
 int res = callbackwriteargs(&client, param, status);
 
 stream.WriteEndTagOMA();
 
 stream.WriteEndTagOMATTR();
 OML_OMOBJ::WriteEndTag(stream);
  res = stream.WriteSCSCPEndMsg();

 SCSCP_getdebugger()<<"\nSCSCP_procedurecompleted_Write() - sortie et retourne "<<res<<"\n";
 return res;
}

/*-----------------------------------------------------------------*/
/*! read the object from the stream
   @param stream (inout) input stream
   @param iter (inout) iterator
   @param mustbecookie (in) = true => must be a cookie
*/
/*-----------------------------------------------------------------*/
bool SCSCP_procedurecompleted::Read(OML_inputstream& stream,  OML_inputstream::iterator& iter, bool mustbecookie)
{
 bool res;
 SCSCP_getdebugger()<<"SCSCP_procedurecompleted::Read() - entree\n";
 
 if (mustbecookie)
 {
  res = OML_OMR::Read(iter, m_cookie);
 }
 else
 {
  res = m_Resultats.Read(stream, iter, "");
 }
 SCSCP_getdebugger()<<"SCSCP_procedurecompleted::Read() - sortie et retourne "<<res<<"\n";
 return res;
}

